package com.tenlionsoft.aimz_k.dao

import androidx.room.Dao
import androidx.room.Insert
import androidx.room.OnConflictStrategy
import androidx.room.Query
import com.tenlionsoft.aimz_k.model.MsgBean
import com.tenlionsoft.aimz_k.model.MsgCategoryBean

@Dao
interface MsgDao {
    /**
     * 更新信息状态
     */
    @Query("UPDATE db_msg SET status = :status WHERE messageId = :messageId")
    suspend fun updateStatus(messageId: String, status: String)

    /**
     * 获取全部信息
     *
     * @return
     */
    @Query("SELECT * FROM db_msg")
    suspend fun getAllMsg(): List<MsgBean?>?

    /**
     * 根据ID获取消息
     *
     * @param id
     * @return
     */
    @Query("SELECT * FROM db_msg WHERE id =(:id) ")
    suspend fun getMsgById(id: String?): List<MsgBean?>?

    /**
     *
     */

    /**
     * 根据信息来源查询消息
     *
     * @param from
     * @return
     */
    @Query("SELECT * FROM db_msg WHERE `senderId`=(:from)")
    suspend fun getMsgByFrom(from: String?): List<MsgBean?>?

    /**
     * 根据发送给谁进行查询
     *
     * @param to
     * @return
     */
    @Query("SELECT * FROM db_msg WHERE `receiverId` =(:to)")
    suspend fun getMsgByTo(to: String?): List<MsgBean?>?


    /**
     * 根据哪里
     *
     * @param form
     * @param pageSize
     * @param page
     * @return
     */
    @Query("SELECT * FROM db_msg WHERE `senderId`=(:form) ORDER BY timestamp LIMIT (((:page)-1)*(:pageSize)),(:pageSize)")
    suspend fun getMsgByPage(form: String?, pageSize: Int, page: Int): List<MsgBean?>?

    /**
     * 查询与单人的聊天记录-分页
     */
    @Query("SELECT * FROM db_msg t1  WHERE t1.id IN (SELECT id FROM db_msg WHERE `senderId`=(:form) AND `receiverId`=(:to) OR `senderId`=(:to) AND `receiverId`=(:form) ORDER BY timestamp DESC LIMIT (((:page)-1)*(:pageSize)),(:pageSize)) ORDER by t1.timestamp ASC")
    suspend fun getMsgByFromOrToPage(
        form: String?, to: String?, pageSize: Int, page: Int
    ): List<MsgBean?>?

    /**
     * 查询与单人的聊天记录-全部
     */
    @Query("SELECT * FROM db_msg t1  WHERE loginUserId=:userId AND t1.id IN (SELECT id FROM db_msg WHERE loginUserId=:userId AND senderId=:form AND receiverId=:to OR senderId=:to AND receiverId=:form ORDER BY timestamp DESC) ORDER by t1.timestamp ASC ")
    suspend fun getMsgByFromOrToPage(
        userId: String,
        form: String,
        to: String,
    ): List<MsgBean>

    /**
     * 清空与某个人的聊天记录
     */
    @Query("DELETE FROM db_msg WHERE loginUserId=:userId AND (senderId=:from AND receiverId=:to) OR (senderId=:from AND senderId=:to)")
    suspend fun delChatHistory(userId: String, from: String, to: String)

    /**
     * 添加消息多个
     *
     * @param beans
     */
    @Insert(onConflict = OnConflictStrategy.REPLACE)
    suspend fun insertAll(vararg beans: MsgBean)

    /**
     * 批量添加
     */
    @Insert(onConflict = OnConflictStrategy.REPLACE)
    suspend fun insertAll(beans: List<MsgBean>)


    /**
     * 添加消息-单个
     *
     * @param bean
     */
    @Insert(onConflict = OnConflictStrategy.REPLACE)
    suspend fun insertMsg(bean: MsgBean)

    /**
     * 清除全部聊天记录
     */
    @Query("DELETE FROM db_msg")
    suspend fun delAllMsg()

    /**
     * 更新 头像和名称
     */
    @Query("UPDATE db_msg SET userNickName=:nickName,avatar=:avatar WHERE receiverId=:toId AND senderId=:fromId")
    suspend fun updateUserInfoById(nickName: String, avatar: String, fromId: String, toId: String)

    /**
     * 更新 头像和名称
     */
    @Query("UPDATE db_msg SET userNickName=:nickName,avatar=:avatar WHERE messageId=:messageId AND loginUserId=:loginUserId")
    suspend fun updateUserInfoByMessageId(
        nickName: String,
        avatar: String,
        messageId: String,
        loginUserId: String
    )

    /**
     * 搜索聊天内容
     */
    @Query(
        "SELECT * FROM db_category  WHERE ((receiverId IN (SELECT receiverId FROM db_msg WHERE body LIKE '%'||:content||'%')) " + "AND (senderId IN (SELECT senderId FROM db_msg WHERE body LIKE '%'||:content||'%'))) " + "OR ((receiverId IN(SELECT senderId FROM db_msg WHERE body LIKE '%'||:content||'%')) " + "AND (senderId IN(select receiverId FROM db_msg where body like '%'||:content||'%')))"
    )
    suspend fun queryContent(content: String): List<MsgCategoryBean>?
}