package com.tenlionsoft.aimz_k.services

import android.annotation.SuppressLint
import android.app.NotificationChannel
import android.app.NotificationManager
import android.app.Service
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.media.MediaPlayer
import android.os.Build
import android.os.IBinder
import android.util.Log
import com.google.gson.Gson
import com.tenlionsoft.aimz_k.socket.WsManager
import com.tenlionsoft.baselib.contacts.NetConfig
import com.tenlionsoft.baselib.contacts.ProjectConfig
import com.tenlionsoft.baselib.utils.SpUtils
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.core.Observable
import io.reactivex.rxjava3.disposables.Disposable
import io.reactivex.rxjava3.schedulers.Schedulers
import java.util.concurrent.TimeUnit


/**
 * Socket service
 */
class SocketService : Service(), WsManager.MsgCallBack {
    private var mWsManager: WsManager? = null
    private val gson = Gson()
    private lateinit var mLocalReceiver: LocalReceiver
    private lateinit var mMediaPlayer: MediaPlayer
    private var msgObserver = arrayListOf<String>()
    private lateinit var mDispose: Disposable
    override fun onCreate() {
        super.onCreate()
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            val channel = NotificationChannel(
                ProjectConfig.NOTIFY_CHANNEL_ID, "Ai秒著客服",
                NotificationManager.IMPORTANCE_DEFAULT
            )
            val manager = getSystemService(NotificationManager::class.java)
            manager.createNotificationChannel(channel)
        }
    }


    @SuppressLint("ForegroundServiceType")
    override fun onStartCommand(intent: Intent?, flags: Int, startId: Int): Int {
//        val builder: Notification.Builder
//        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
//            builder = Notification.Builder(this, ProjectConfig.NOTIFY_CHANNEL_ID)
//        } else {
//            builder = Notification.Builder(this)
//        }
//        builder.setContentTitle("Ai秒著")
//            .setContentText("聊天已经登陆")
//            .setSmallIcon(R.drawable.app_logo_small)
//        val notification: Notification = builder.build()
//        startForeground(startId, notification)

        this.startSocket() //开启Socket
        mMediaPlayer = MediaPlayer.create(this, com.tenlionsoft.baselib.R.raw.chat)
        registerLocalReceiver()
        return START_REDELIVER_INTENT
    }


    @SuppressLint("UnspecifiedRegisterReceiverFlag")
    private fun registerLocalReceiver() {
        val intentFilter = IntentFilter()
        mLocalReceiver = LocalReceiver()
        intentFilter.addAction(ProjectConfig.A_S_MSG_SEND)//发送消息
        intentFilter.addAction(ProjectConfig.A_S_LOGOUT)  //断开socket
        intentFilter.addAction(ProjectConfig.A_S_RE_LOGIN)//重新连接socket
        registerReceiver(mLocalReceiver, intentFilter)
    }

    /**
     * 开启链接socket
     */
    private fun startSocket() {
        if (mWsManager == null) {
            mWsManager = WsManager.Builder(applicationContext)
                .wsUrl(NetConfig.WS_URL + "?token=" + SpUtils.getToken())
                .needReconnect(true)
                .addCallBack(this)
                .build()
        }
        if (!mWsManager!!.isWsConnected()) {
            mWsManager!!.startConnect()
            subscribeMsgChange()
        }

    }

    private fun stopSocket() {
        if (mWsManager != null && mWsManager!!.isWsConnected()) {
            mWsManager!!.stopConnect()
            mWsManager = null
        }
    }

    override fun onBind(intent: Intent?): IBinder? {
        return null
    }

    override fun onDestroy() {
        Log.e("SocketService", "onDestroy服务销毁")
        stopSocket()
        unregisterReceiver(mLocalReceiver)
        mMediaPlayer.release()
        if (!mDispose.isDisposed) {
            mDispose.dispose()
        }
        super.onDestroy()
    }

    //发送消息
    inner class LocalReceiver : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent?) {
//            val isConnect = mWsManager?.isNetworkConnected(context)
//            if (true == isConnect) {
            when (intent?.action) {
                //发送消息
                ProjectConfig.A_S_MSG_SEND -> {
                    val msgConvertBean = intent.getSerializableExtra("msgBean")
                    val msg = gson.toJson(msgConvertBean)
                    Log.e("SocketService", "发送消息:${msg}")
                    mWsManager?.sendMessage(msg)
                }
                //断开连接
                ProjectConfig.A_S_LOGOUT -> {
                    stopSocket()
                    stopSelf()
                }
                //重新连接
                ProjectConfig.A_S_RE_LOGIN -> {
                    stopSocket()
                    startSocket()
                }
            }
//            } else {
//                ToastUtils.error("请检查网络")
//            }

        }

    }


    private fun subscribeMsgChange() {
        mDispose = Observable.interval(1500, TimeUnit.MILLISECONDS)
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())
            .subscribe {
                if (msgObserver.isNotEmpty()) {
                    if (msgObserver.size > 1) {
                        val intent = Intent(ProjectConfig.A_S_MSG_MULTI)
                        intent.putStringArrayListExtra("msg", msgObserver)
                        sendBroadcast(intent)
                    } else {
                        val intent = Intent(ProjectConfig.A_S_MSG_RECEIVER)
                        intent.putExtra("msg", msgObserver[0])
                        sendBroadcast(intent)
                        if (!msgObserver[0].contains("STATUS")) {
                            mMediaPlayer.start()
                        }
                    }
                    msgObserver.clear()
                }
            }
    }

    //socket接收到信息
    override fun onCallBackMsg(str: String) {
        Log.e("SocketService", "接收信息:${str}")
        msgObserver.add(str)
    }
}