package com.tenlionsoft.aimz_k.viewmodel

import android.app.Application
import android.content.Context
import android.content.Intent
import android.util.Log
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.google.gson.Gson
import com.tenlionsoft.aimz_k.ConvertBeanUtils
import com.tenlionsoft.aimz_k.adapter.CategoryAdapter
import com.tenlionsoft.aimz_k.dao.DbManager
import com.tenlionsoft.aimz_k.model.BodyContent
import com.tenlionsoft.aimz_k.model.CoverSealedBean
import com.tenlionsoft.aimz_k.model.MsgCategoryBean
import com.tenlionsoft.aimz_k.model.MsgConvertBean
import com.tenlionsoft.aimz_k.model.Receiver
import com.tenlionsoft.aimz_k.model.Sender
import com.tenlionsoft.aimz_k.model.UserInfoBean
import com.tenlionsoft.aimz_k.net.BaseApi
import com.tenlionsoft.baselib.base.BaseViewModel
import com.tenlionsoft.baselib.contacts.ProjectConfig
import com.tenlionsoft.baselib.net.RetrofitClient
import com.tenlionsoft.baselib.utils.SpUtils
import com.tenlionsoft.baselib.utils.TimeUtils
import com.tenlionsoft.baselib.utils.ToastUtils
import com.tenlionsoft.baselib.widget.AdapterItemClickListener
import com.tenlionsoft.baselib.widget.AdapterItemLongClickListener
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

class MsgViewModel(val context: Application) : BaseViewModel() {
    private val _pwdList = MutableLiveData<List<MsgCategoryBean>>()
    var adapter: CategoryAdapter = CategoryAdapter(
        _pwdList.value
            ?: emptyList(), this
    )
    var onItemClickListener: AdapterItemClickListener<MsgCategoryBean>? = null
    var onItemLongClickListener: AdapterItemLongClickListener<MsgCategoryBean>? = null
    val isLogining = MutableLiveData(false)
    private val mGson = Gson()
    private var mCurrentPage: Int = 1
    var isHasMore: MutableLiveData<Boolean> = MutableLiveData<Boolean>(false)
    private val retrofitClient = RetrofitClient.getInstance(context)
    private val baseApi = retrofitClient.create(BaseApi::class.java)
    private var userInfo: UserInfoBean? = null

    /**
     * 获取列表数据
     */
    private fun getList(isFirst: Boolean) {
        viewModelScope.launch {
            if (isFirst) {
                _pwdList.value = emptyList()
                mCurrentPage = 1
            } else {
                ++mCurrentPage
            }
            val list = getCategoryListByPage(mCurrentPage)
            isHasMore.value = list.isEmpty()
            val updateData: List<MsgCategoryBean> = _pwdList.value!! + list
            _pwdList.value = updateData
            adapter.setData(_pwdList.value!!)
        }
    }

    private suspend fun getCategoryListByPage(page: Int): List<MsgCategoryBean> {
        var tempList: List<MsgCategoryBean>
        withContext(Dispatchers.IO) {
            val cDao = DbManager.db.categoryDao()
            tempList = cDao.getCategoryPage(SpUtils.getId(), 10, page)
        }
        return tempList
    }

    /**
     * 加载更多
     */
    fun loadMore() {
        getList(false)
    }

    /**
     * 登陆websocket
     */
    fun doLoginWebsocket() {
        isLogining.value = true
    }

    //接收到信息
    fun fromMsg(msg: String) {
        if (msg.isNotEmpty()) {
            when (val bean = ConvertBeanUtils.covertBean(msg)) {
                //状态消息
                is CoverSealedBean.StateBean -> {
                    viewModelScope.launch {
                        withContext(Dispatchers.IO) {
                            val dao = DbManager.db.msgDao()
                            val cDao = DbManager.db.categoryDao()
                            val statusType =
                                mGson.fromJson(bean.data.body, BodyContent::class.java).statusType!!
                            dao.updateStatus(bean.data.messageId, statusType)
                            cDao.updateStatus(bean.data.sender.senderId!!, statusType)
                        }
                        //刷新
                        refresh()
                    }
                }
                //消息
                is CoverSealedBean.Msg -> {
                    //插入信息表
                    //并更新category表
                    Log.e("MsgViewModel", "fromMsg:${bean.msgBean.loginUserId}")
                    viewModelScope.launch {
                        withContext(Dispatchers.IO) {
                            val msgDao = DbManager.db.msgDao()
                            val cDao = DbManager.db.categoryDao()
                            msgDao.insertMsg(bean.msgBean)
                            cDao.updateOrInsert(bean.msgBean.senderId!!, bean.msgBean)
                        }
                        doGetUserInfo(bean.msgBean.messageId!!, bean.msgBean.senderId!!)
                        sendStatusMsg(bean.msgBean.messageId!!, bean.msgBean.senderId!!, context)
                        //刷新列表
                        refresh()
                    }
                }
            }
        }
    }

    //接收到多条消息
    fun fromMultiMsg(msg: ArrayList<String>) {
        val tempList = arrayListOf<CoverSealedBean>()
        msg.forEach {
            val b = ConvertBeanUtils.covertBean(it)
            tempList.add(b)
        }
        viewModelScope.launch {
            withContext(Dispatchers.IO) {
                val msgDao = DbManager.db.msgDao()
                val cDao = DbManager.db.categoryDao()
                tempList.forEach {
                    when (it) {
                        //状态消息
                        is CoverSealedBean.StateBean -> {

                            val statusType =
                                mGson.fromJson(it.data.body, BodyContent::class.java).statusType!!
                            msgDao.updateStatus(it.data.messageId, statusType)
                            cDao.updateStatus(it.data.sender.senderId!!, statusType)
                        }
                        //消息
                        is CoverSealedBean.Msg -> {
                            //插入信息表
                            //并更新category表
                            msgDao.insertMsg(it.msgBean)
                            cDao.updateOrInsert(it.msgBean.senderId!!, it.msgBean)
                            doGetUserInfo(it.msgBean.messageId!!, it.msgBean.senderId!!)
                            sendStatusMsg(
                                it.msgBean.messageId!!,
                                it.msgBean.senderId!!,
                                context
                            )
                        }
                    }
                }
                refresh()
            }
        }
    }


    //获取用户头像和名称
    private fun doGetUserInfo(messageId: String, userId: String) {
        viewModelScope.launch {
            try {
                val userInfoBean = retrofitClient.makeApiCall {
                    baseApi.getUserInfo(userId)
                }
                userInfo = userInfoBean
                Log.e("MsgViewModel", "doGetUserInfo:${userInfo}");
                withContext(Dispatchers.IO) {
                    //更新数据
                    val cDao = DbManager.db.categoryDao()
                    val mDao = DbManager.db.msgDao()
                    if (userInfoBean != null) {
                        cDao.updateInfoByMessageId(
                            userInfoBean.nickname,
                            userInfoBean.avatar,
                            messageId,
                            SpUtils.getId()
                        )
                        mDao.updateUserInfoByMessageId(
                            userInfoBean.nickname,
                            userInfoBean.avatar,
                            messageId,
                            SpUtils.getId()
                        )
                    }
                }
                refresh()
            } catch (e: Exception) {
                e.printStackTrace()
            }
        }
    }

    private fun sendStatusMsg(msgId: String, fromId: String, context: Context) {
        val b = buildStatusBean(msgId, fromId)
        val intent = Intent(ProjectConfig.A_S_MSG_SEND)
        intent.putExtra("msgBean", b)
        context.sendBroadcast(intent)
    }

    //构建状态信息
    private fun buildStatusBean(messageId: String, fromId: String): MsgConvertBean {
        val bodyBean = BodyContent(content = null, msg = "", statusType = "SUCCESS_RECEIVED")
        val body = mGson.toJson(bodyBean)
        return MsgConvertBean(
            body = body,
            customMessageType = "",
            messageId = messageId,
            messageType = ProjectConfig.MSG_STATUS,
            metadata = "",
            timestamp = TimeUtils.getNowDateMillis(),
            sender = Sender(
                senderId = SpUtils.getId(),
                senderType = ""
            ),
            receiver = Receiver(
                receiverId = fromId,
                receiverType = "SINGLE_USER"
            ),
            status = "",
        )
    }

    /**
     * 刷新
     */
    fun refresh() {
        getList(true)
    }

    //删除聊天
    fun doDelChat(d: MsgCategoryBean) {
        viewModelScope.launch {
            showLoadDialog.value = true
            withContext(Dispatchers.IO) {
                val cDao = DbManager.db.categoryDao()
                val mDao = DbManager.db.msgDao()
                cDao.delChatBySenderId(SpUtils.getId(), d.senderId!!)
                mDao.delChatHistory(SpUtils.getId(), d.senderId!!, d.receiverId!!)
            }
            showLoadDialog.value = false
            ToastUtils.success("删除成功")
            refresh()
        }
    }


}